package com.dsic.gj_erp.service.jhgk.impl;


import cn.hutool.core.date.DateUtil;
import cn.hutool.core.io.FileUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.dsic.gj_erp.bean.jhgk.DmXbjh;
import com.dsic.gj_erp.mapper.jhgk.DmXbjhMapper;
import com.dsic.gj_erp.service.jhgk.DmXbjhService;
import com.dsic.gj_erp.util.ExcelUtil;
import org.apache.commons.lang3.StringUtils;
import org.modelmapper.ModelMapper;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import java.io.File;
import java.io.IOException;
import java.util.*;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.stream.Collectors;

/**
 * <p>
 * 服务实现类
 * </p>
 *
 * @author xn
 * @since 2023-09-15
 */
@Service
public class DmXbjhServiceImpl extends ServiceImpl<DmXbjhMapper, DmXbjh> implements DmXbjhService {
    @Resource
    DmXbjhMapper dmXbjhMapper;
    @Resource
    ModelMapper modelMapper;

    @Value("${xbjh-path}")
    String xbjhPath;

    @Override
    public Map<String, List<DmXbjh>> getList(JSONObject json) {

        QueryWrapper<DmXbjh> queryWrapper = new QueryWrapper<>();
        if (StringUtils.isNotEmpty(json.getString("dcCh"))) {
            queryWrapper.eq("dc_ch", json.getString("dcCh"));
        }
        if (StringUtils.isNotEmpty(json.getString("Zt"))) {
            queryWrapper.eq("zt", json.getString("Zt"));
        }
        String startTime=null;
        String endTime=null;
        if (StringUtils.isNotEmpty(json.getString("nf"))) {
            String nf = json.getString("nf");
            startTime = nf.split("~")[0];
            endTime = nf.split("~")[1];
            queryWrapper.between("nf", startTime, endTime);
        }
        queryWrapper.orderByDesc("zl");
        String version=json.getString("version");
        if (version!=null){
            version=version.replace("v","");
        }
        List<DmXbjh> dmTzjhs = dmXbjhMapper.getList(json.getString("dcCh"), json.getString("nf"),startTime,endTime,version);
        Map<String, List<DmXbjh>> map = dmTzjhs.stream()
                .collect(Collectors.groupingBy(DmXbjh::getDcCh));

        //对map的value进行排序，并返回降序以后的map集合
        LinkedHashMap<String, List<DmXbjh>> collect = map.entrySet().stream().sorted((entry1, entry2) -> {
            Double zl1 = entry1.getValue().stream().filter(l -> ObjectUtil.isNotEmpty(l.getZl())).mapToDouble(DmXbjh::getZl).sum();
            Double zl2 = entry2.getValue().stream().filter(l -> ObjectUtil.isNotEmpty(l.getZl())).mapToDouble(DmXbjh::getZl).sum();
            //降序排序
            return zl2.compareTo(zl1);
        }).collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue, (oldValue, newValue) -> oldValue, LinkedHashMap::new));

        return collect;
    }

    public Map<String, List<DmXbjh>> getByVersion(String startTime, String _version) {
        List<HashMap<String, Object>> version = this.getVersion(startTime);
        if (ObjectUtil.isNotEmpty(version) && !version.get(version.size() - 1).get("version").equals(_version)) {
            List<DmXbjh> dmXbjhs = new ArrayList<>();
            for (HashMap<String, Object> map : version) {
                if (map.get("version").equals(_version)) {
                    String fileName = (String) map.get("fileName");
                    File file = FileUtil.file(xbjhPath, fileName);
                    if (file.exists()) {
                        JSONObject jsonObject = this.readFile(file);
                        for (String key : jsonObject.keySet()) {
                            JSONArray jsonArray = jsonObject.getJSONArray(key);
                            for (Object object : jsonArray) {
                                DmXbjh dmXbjh = modelMapper.map(object, DmXbjh.class);
                                dmXbjh.setDcCh(key);
                                dmXbjhs.add(dmXbjh);
                            }
                        }
                    }
                }
            }
            Map<String, List<DmXbjh>> map = dmXbjhs.stream()
                    .collect(Collectors.groupingBy(DmXbjh::getDcCh));

            //对map的value进行排序，并返回降序以后的map集合
            LinkedHashMap<String, List<DmXbjh>> collect = map.entrySet().stream().sorted((entry1, entry2) -> {
                Double zl1 = entry1.getValue().stream().filter(l -> ObjectUtil.isNotEmpty(l.getZl())).mapToDouble(DmXbjh::getZl).sum();
                Double zl2 = entry2.getValue().stream().filter(l -> ObjectUtil.isNotEmpty(l.getZl())).mapToDouble(DmXbjh::getZl).sum();
                //降序排序
                return zl2.compareTo(zl1);
            }).collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue, (oldValue, newValue) -> oldValue, LinkedHashMap::new));
            return collect;
        }
        return null;
    }

    public List<HashMap<String, Object>> getVersion(String nf) {
        if (nf == null) {
            return null;
        }
        File[] files = FileUtil.ls(xbjhPath);
        List<HashMap<String, Object>> versions = new ArrayList<>();
        for (File file : files) {
            String name = FileUtil.getPrefix(file.getName());
            String[] s = name.split("_");
            if (nf.equals(s[0])) {
                HashMap<String, Object> map = new HashMap<>();
                map.put("fileName", file.getName());
                map.put("version", s[1]);
                versions.add(map);
            }
        }
        return versions;
    }

    public JSONObject readFile(File file) {
        return ExcelUtil.readDmXbjh(file);
    }

    @Override
    @Transactional
    public JSONObject upload(MultipartFile file) throws IOException {
        JSONObject dmXbjhs = ExcelUtil.readDmXbjh(file);
        if (dmXbjhs != null) {
            File[] files = FileUtil.ls(xbjhPath);
            AtomicInteger maxVersion = new AtomicInteger(0);
            if (files.length > 0) {
                HashSet<String> names = new HashSet<>();
                for (File file1 : files) {
                    String name = FileUtil.getPrefix(file1.getName());
                    if (name.contains(DateUtil.thisYear()+"")){
                        String[] s = name.split("_");
                        names.add(s[s.length - 1]);
                    }
                }

                names.stream()
                        .max(String::compareTo)
                        .ifPresent(item -> {
                    String v = item.replace("v", "");
                    maxVersion.set(Integer.parseInt(v));
                });
            }

            String type = FileUtil.extName(file.getOriginalFilename());
            File uploadFile = new File(xbjhPath);
            if (!uploadFile.exists()) {
                uploadFile.mkdirs();
            }
            int version = maxVersion.incrementAndGet();
            File uploadFiles = new File(StrUtil.format("{}/{}_{}{}{}{}", xbjhPath, DateUtil.thisYear(), "v", version, StrUtil.DOT, type));
            file.transferTo(uploadFiles);
            this.save(dmXbjhs,version);
        }
        return dmXbjhs;
    }

    @Override
    @Transactional
    public void save(JSONObject json,int version) {
        List<DmXbjh> dmXbjhs = new ArrayList<>();
        for (String key : json.keySet()) {
            JSONArray jsonArray = json.getJSONArray(key);
            for (Object object : jsonArray) {
                //  JSONObject dmxb=JSONObject.parseObject(object.toString());
                DmXbjh dmXbjh = modelMapper.map(object, DmXbjh.class);
                dmXbjh.setDcCh(key);
                dmXbjh.setVersion(version);
                dmXbjhs.add(dmXbjh);
//                this.remove(new QueryWrapper<DmXbjh>().eq("dc_ch", dmXbjh.getDcCh())
//                        .eq("nf", dmXbjh.getNf())
//                        .eq("yf", dmXbjh.getYf())
//                );
            }
        }

        saveBatch(dmXbjhs);
    }

    @Override
    public void tj(List<DmXbjh> dmXbjhs) {
        dmXbjhs.forEach(dmXbjh -> {
            dmXbjh.setZt("1");
        });
        updateBatchById(dmXbjhs);
    }
}
