package com.dsic.gj_erp.service.zyjh;

import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.io.FileUtil;
import cn.hutool.core.util.ObjUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.dsic.gj_erp.bean.jcsj.DmBzry;
import com.dsic.gj_erp.bean.jcsj.DmCbxxp;
import com.dsic.gj_erp.bean.jhgk.DmYdjh;
import com.dsic.gj_erp.bean.jhgk.DmYdjhLj;
import com.dsic.gj_erp.bean.zyjh.DmYppyz;
import com.dsic.gj_erp.controller.zyjh.dto.PgWithPcDto;
import com.dsic.gj_erp.controller.zyjh.dto.ShangLiao;
import com.dsic.gj_erp.controller.zyjh.dto.SljhSearch;
import com.dsic.gj_erp.controller.zyjh.dto.YppDto;
import com.dsic.gj_erp.mapper.jhgk.DmYdjhMapper;
import com.dsic.gj_erp.service.jcsj.DmBzryService;
import com.dsic.gj_erp.service.jcsj.DmCbxxpService;
import com.dsic.gj_erp.service.jhgk.DmYdjhLjService;
import com.dsic.gj_erp.util.ExcelUtil;
import com.google.common.collect.ImmutableMap;
import lombok.RequiredArgsConstructor;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.io.File;
import java.util.List;
import java.util.Map;
import java.util.regex.Pattern;
import java.util.stream.Collectors;

@Service
@RequiredArgsConstructor
public class ZyjhService extends ServiceImpl<DmYdjhMapper,DmYdjh> {

    private final DmBzryService bzryService;
    private final DmCbxxpService dmCbxxpService;
    private final RedisTemplate<String,Object> redisTemplate;
    private final DmYdjhLjService ljService;
    private final DmYppyzService yppyzService;

    private static final String MPDHSJ_KEY="MPDHSJ";

    /**
     * 获取带单据号的上料计划,单据号从pdf二维码表中查询
     */
    public List<DmYdjh> getSljhWithDjh(SljhSearch search){
        return baseMapper.getSljhWithDjh(search);
    }

    public void ppbg(Integer id){
        yppyzService.update(new UpdateWrapper<DmYppyz>().set("status","已配送").eq("id",id));
    }

    public List<YppDto> getYbf(DmYppyz yppyz) {
        List<YppDto> yppyzList = yppyzService.bgList(yppyz);

        return yppyzList.stream().filter(item->{
            List<DmYdjhLj> ljList = item.getLjList();
            for (DmYdjhLj lj :ljList){
                if ("01".equals(lj.getPpzt())){
                    return false;
                }
            }
            return true;
        }).collect(Collectors.toList());
    }

    public void ppwc(Integer id,String dcCh,String dcPl,String dcFd){
        int count = baseMapper.getNotPPLjCount(dcCh,dcPl,dcFd);
        if (count==0){
            yppyzService.update(new UpdateWrapper<DmYppyz>().set("status","已摆放").eq("id",id));
        }
    }

    public void ppBGong(List<DmYdjhLj> list){
        ljService.updateBatchById(list);
    }

    public void ppBGong(List<DmYdjhLj> list,String dcCh,String dcPl,String zyq){
        ljService.updateBatchById(list);
    }

    /**
     * 获取预配盘原则和对应未配盘零件信息
     */
    public Object getWppljAndPpyz(String zyq1){
        return baseMapper.getWppljAndPpyz(zyq1);
    }

    public Map<String,Object> getMPSYCLJLJBP(String dcCh,String tzbh){
        return baseMapper.getMPSYCLJLJBP(dcCh,tzbh);
    }

    public String getMpDhsjDhh(){
        String date=DateUtil.date().toString("yyMMdd");
        Object xh = redisTemplate.opsForHash().get(MPDHSJ_KEY, date);
        if(ObjUtil.isEmpty(xh)){
            redisTemplate.opsForHash().put(MPDHSJ_KEY, date,0);
        }
        Long increment = redisTemplate.opsForHash().increment(MPDHSJ_KEY, date, 1);
        return date+StrUtil.padPre(increment+"",3,"0");
    }

    public String getMpDhsjDhh(String date){
        Object xh = redisTemplate.opsForHash().get(MPDHSJ_KEY, date);
        if(ObjUtil.isEmpty(xh)){
            redisTemplate.opsForHash().put(MPDHSJ_KEY, date,0);
        }
        Long increment = redisTemplate.opsForHash().increment(MPDHSJ_KEY, date, 1);
        return date+StrUtil.padPre(increment+"",3,"0");
    }

    @Transactional(rollbackFor = Exception.class)
    public List<DmYdjhLj> createLj1(String dcCh, String dcPl, String tzbh){
        ljService.remove(Wrappers.<DmYdjhLj>lambdaQuery()
                .eq(DmYdjhLj::getCzbh,dcCh)
                .eq(DmYdjhLj::getPl,dcPl)
                .eq(DmYdjhLj::getTlth,tzbh)
        );
        List<DmYdjhLj> ljInfo = baseMapper.getLjInfo(dcCh, dcPl, tzbh);
        ljService.saveBatch(ljInfo);
        return ljInfo;
    }

    @Transactional(rollbackFor = Exception.class)
    public List<DmYdjhLj> createLj(String dcCh, String dcPl, String tzbh){

        List<DmYdjhLj> ljInfo = baseMapper.getLjInfo(dcCh, dcPl, tzbh);
        //给零件设置流向
        Map<String, List<DmYdjhLj>> _ljMap = ljInfo.stream()
                .collect(Collectors
                        .groupingBy(item ->
                                item.getCzbh() + "_" + item.getPl() + "_" + item.getFd()
                        ));
        _ljMap.forEach((key,values)->{
            try  {
                String[] keys=key.split("_");
                String ch=keys[0];
                String pl=keys[1];
                String fd=keys[2];
                File dir = new File("static/upload/" + ch + "/" + pl + "/其他");
                String patternStr = pl + "_" + fd + "_分段配套表";
                List<File> fileList = FileUtil.loopFiles(dir);
                fileList.stream()
                        .filter(file -> file.getName().contains(patternStr))
                        .forEach(file -> {
                            if (file.exists()){
                                List<String> strings = ExcelUtil.ReadExcelByLine(file);
                                for (DmYdjhLj _lj : values){
                                    for (String _str:strings){
                                        if (_str.contains(_lj.getTlth())&&_str.contains(_lj.getLjbh())){
                                            String[] _strArr=_str.split(" ");
                                            if (_strArr.length>8){
                                                _lj.setTz(_strArr[7]);
                                                _lj.setLx(_strArr[8]);
                                            }
                                            if (_strArr.length>9){
                                                _lj.setXj(_strArr[9]);
                                            }
                                        }
                                    }
                                }
                            }
                        });
            } catch (Exception e) {
                e.printStackTrace();
            }
        });

        ljService.remove(Wrappers.<DmYdjhLj>lambdaQuery()
                .eq(DmYdjhLj::getCzbh,dcCh)
                .eq(DmYdjhLj::getPl,dcPl)
                .eq(DmYdjhLj::getTlth,tzbh)
        );
        ljService.saveBatch(ljInfo);

        //自动生成无摆放位置的预配盘原则
        //2050401取消配盘原则自动生成功能,改为手动设置
//        Map<String, List<DmYdjhLj>> collect = ljInfo.stream()
//                .collect(Collectors
//                        .groupingBy(item ->
//                                item.getCzbh() + " "
//                                        + item.getPl() + " "
//                                        + item.getFd() + " "
//                                        + item.getZl() + " "
//                                        + (StrUtil.isNotEmpty(item.getLx())?item.getLx():"")
//                        ));
//
//        collect.keySet().forEach(item -> {
//            DmYppyz dmYppyz = DmYppyz.of(item);
//            if (dmYppyz!=null){
//                int count = yppyzService.count(Wrappers.<DmYppyz>lambdaQuery()
//                        .eq(DmYppyz::getDcCh, dmYppyz.getDcCh())
//                        .eq(DmYppyz::getDcPl, dmYppyz.getDcPl())
//                        .eq(DmYppyz::getDcFd, dmYppyz.getDcFd())
//                        .eq(DmYppyz::getZl, dmYppyz.getZl())
//                        .eq(StrUtil.isNotEmpty(dmYppyz.getLx()),DmYppyz::getLx, dmYppyz.getLx())
//                );
//                if (count==0){
//                    yppyzService.save(dmYppyz);
//                }
//            }
//        });

        return ljInfo;
    }

    public List<DmYdjh> getYdjhByZtOrForeman(Integer zt,String zyq){
        return baseMapper.getWithDjh(zt,zyq);
    }

    public List<DmYdjh> getYdjhByZtOrForeman(Integer zt,String foreman,String userCode,String zyq){
        QueryWrapper<DmYdjh> wrapper = new QueryWrapper<DmYdjh>()
                .ge("zt",11)
                .ge(ObjUtil.isNotEmpty(zt),"zt",zt)
                .eq(StrUtil.isNotEmpty(zyq),"zyq1",zyq);

        switch (zt){
            case 31:
//                wrapper.eq(!"1".equals(foreman),"pwpgry",foreman);
//                wrapper.eq("1".equals(foreman),"pwpgry",userCode);
                wrapper.isNull("pwfkry");
                break;
            case 51:
//                wrapper.eq(!"1".equals(foreman),"hxpgry",foreman);
//                wrapper.eq("1".equals(foreman),"hxpgry",userCode);
                wrapper.isNull("hxfkry");
                break;
            case 61:
//                wrapper.eq(!"1".equals(foreman),"qgpgry",foreman);
//                wrapper.eq("1".equals(foreman),"qgpgry",userCode);
                wrapper.isNull("qgfkry");
                break;
        }
        wrapper.orderByDesc("dc_ch","dc_pl,tzbh");

        return list(wrapper);
    }

    public List<Map<String,Object>> getYdjhByZt(Integer zt,String wgzt,String wclzt,String zyq){
        DateTime date = DateUtil.date();
        String from = DateUtil.beginOfMonth(date).toString("yyyy/MM/dd");
        String to = DateUtil.endOfMonth(date).toString("yyyy/MM/dd");
        return baseMapper.getYdjhByZt(zyq,zt,wgzt,wclzt,from,to);
    }


    public void pgWithForeman(PgWithPcDto dto){
        LambdaUpdateWrapper<DmYdjh> wrapper = new LambdaUpdateWrapper<DmYdjh>()
                .eq(DmYdjh::getDcCh,dto.getDcCh())
                .eq(DmYdjh::getDcPl,dto.getDcPl())
                .eq(DmYdjh::getZyq1,dto.getZyq());

        String pgrq = DateUtil.date().toString("yyyy/MM/dd HH:mm:ss");
        switch (dto.getZt()){
            case "21":
                wrapper.set(DmYdjh::getPwpgrq,pgrq).set(DmYdjh::getPwpgry,dto.getForemanId());
                wrapper.lt(DmYdjh::getZt,31);//仅处理未派工的数据
                break;
            case "42":
            case "52":
                wrapper.ge(DmYdjh::getZt,32);
                if ("HXPG".equals(dto.getZxZtName())){
                    wrapper.set(DmYdjh::getHxpgrq,pgrq).set(DmYdjh::getHxpgry,dto.getForemanId()).set(DmYdjh::getZt,"51");
                    wrapper.isNull(DmYdjh::getHxpgrq).isNull(DmYdjh::getHxpgry);
                    //wrapper.lt(DmYdjh::getZt,51);//仅处理未派工的数据
                }
                if ("QGPG".equals(dto.getZxZtName())){
                    wrapper.set(DmYdjh::getQgpgrq,pgrq).set(DmYdjh::getQgpgry,dto.getForemanId()).set(DmYdjh::getZt,"61");
                    wrapper.eq(DmYdjh::getZt,"52").isNull(DmYdjh::getQgpgrq).isNull(DmYdjh::getQgpgry);
                    //wrapper.lt(DmYdjh::getZt,61);//仅处理未派工的数据
                }
                break;
        }

        this.update(wrapper);
    }

    public DmBzry getForemanByWorker(String worker){
        DmBzry one = bzryService.getOne(Wrappers.<DmBzry>lambdaQuery().eq(DmBzry::getUserCode, worker));
        if ("1".equals(one.getForeman())){
            return one;
        }
        return bzryService.getOne(Wrappers.<DmBzry>lambdaQuery().eq(DmBzry::getUserCode, one.getForeman()));
    }

    public List<DmBzry> getForeman(String zyq){
        return bzryService.list(Wrappers.<DmBzry>lambdaQuery()
                .eq(DmBzry::getForeman, "1").eq(DmBzry::getZyq, zyq));
    }

    public List<DmBzry> getBzryByBz(String zyq,String bz){
        return bzryService.list(Wrappers.<DmBzry>lambdaQuery()
                .eq(DmBzry::getResourcesCode, bz).eq(DmBzry::getZyq, zyq).orderByAsc(DmBzry::getUserName));
    }
    /**
     * 获取所有未处理批次
     */
    public List<Map<String,Object>> getWcl(){
        return baseMapper.getWcl();
    }

    public List<Map<String, Object>> gcjd(){
        ImmutableMap<String, Boolean> map = ImmutableMap.of("flag", true);
        List<DmCbxxp> list = dmCbxxpService.getList(map);
        List<String> czbhList=list.stream().map(DmCbxxp::getCbmc).collect(Collectors.toList());
        return baseMapper.gcjd(czbhList);
    }

    public List<Map<String, Object>> gxjhwcqk(){
        DateTime date = DateUtil.date();
        String from= DateUtil.beginOfMonth(date).toString("yyyy/MM/dd");
        String to =DateUtil.endOfMonth(date).toString("yyyy/MM/dd");
        return baseMapper.gxjhwcqk(from,to);
    }

    public List<Map<String, Object>> gyfcl(){
        return baseMapper.gyfcl();
    }

    public List<Map<String, Object>> jhwcqk(){
        DateTime date = DateUtil.date();
        String from= DateUtil.beginOfMonth(date).toString("yyyy/MM/dd");
        String to =DateUtil.endOfMonth(date).toString("yyyy/MM/dd");
        return baseMapper.jhwcqk(from,to);
    }

    public Map<String,Object> gekuachaliang(){
        DateTime date = DateUtil.date();
        String from= DateUtil.beginOfMonth(date).toString("yyyy/MM/dd");
        String to =DateUtil.endOfMonth(date).toString("yyyy/MM/dd");
        return baseMapper.gekuachaliang(from,to,"11");
    }

    public void updateForSl(ShangLiao sl){
        String date=DateUtil.date().toString("yyyy/MM/dd");
        LambdaUpdateWrapper<DmYdjh> wrapper = Wrappers.<DmYdjh>lambdaUpdate()
                //上料
                .set(ObjUtil.isNotEmpty(sl.getYdjh().getSlycldw()),DmYdjh::getSlycldw, sl.getYdjh().getSlycldw())
                .set(ObjUtil.isNotEmpty(sl.getYdjh().getSlyclcs()),DmYdjh::getSlyclcs, sl.getYdjh().getSlyclcs())
                .set(ObjUtil.isNotEmpty(sl.getYdjh().getSlfkry()),DmYdjh::getSlfkrq, date)
                .set(ObjUtil.isNotEmpty(sl.getYdjh().getSlfkry()),DmYdjh::getSlfkry, sl.getYdjh().getSlfkry())

                //理料
                .set(ObjUtil.isNotEmpty(sl.getYdjh().getLldw()),DmYdjh::getLldw, sl.getYdjh().getLldw())
                .set(ObjUtil.isNotEmpty(sl.getYdjh().getLlcs()),DmYdjh::getLlcs, sl.getYdjh().getLlcs())
                .set(ObjUtil.isNotEmpty(sl.getYdjh().getLlfkry()),DmYdjh::getLlfkrq, date)
                .set(ObjUtil.isNotEmpty(sl.getYdjh().getLlfkry()),DmYdjh::getLlfkry, sl.getYdjh().getLlfkry())
                .eq(DmYdjh::getId, sl.getYdjh().getId());

        switch (sl.getType()){
            case YCL:
                wrapper.set(DmYdjh::getZt, DmYdjh.getSlZt());
                break;
            case LLJ:
                wrapper.set(DmYdjh::getZt, DmYdjh.getLLZt());
        }


        this.update(wrapper);
    }

    private static final String KEY="gxControl";

    public Object cacheable(String gx){
        if (gx!=null){
            redisTemplate.opsForValue().set(KEY,gx);
            return gx;
        }else{
            return redisTemplate.opsForValue().get(KEY);
        }
    }
}
